@@ -6,14 +6,14 @@ use vars qw($VERSION @ISA @EXPORT_OK %EXPORT_TAGS
 				$SRSTAG $SRSWRAP $SRSSEP);
 use Exporter;
 use Carp;
-use Digest::HMAC_MD5;
+use Digest::HMAC_SHA1;
 
-$VERSION = "0.15";
+$VERSION = "0.16";
 @ISA = qw(Exporter);
 
 $SRSTAG = "SRS0";
 $SRSWRAP = "SRS1";
-$SRSSEP = "+";
+$SRSSEP = "=";
 
 @EXPORT_OK = qw($SRSTAG $SRSWRAP $SRSSEP);
 %EXPORT_TAGS = (
@@ -28,8 +28,10 @@ Mail::SRS - Interface to Sender Rewriting Scheme
 
 	use Mail::SRS;
 	my $srs = new Mail::SRS(
-		Secret    => [ .... ],    # scalar or array
-		MaxAge    => 49,          # days
+		Secret     => [ .... ],    # scalar or array
+		MaxAge     => 49,          # days
+		HashLength => 4,           # base64 characters: 4 x 6bits
+		HashMin    => 4,           # base64 characters
 			);
 	my $srsaddress = $srs->forward($sender, $alias);
 	my $sender = $srs->reverse($srsaddress);
@@ -58,20 +60,16 @@ For a better understanding of this code and how it functions, please
 run the interactive walkthrough in eg/simple.pl in this distribution.
 To run this from the build directory, type "make teach".
 
-=head1 WARNING
+=head1 WARNING: MAJOR CHANGES since v0.15
 
-This API is now a release candidate and should remain stable.
+=item The separator character is now =.
 
-There have been INTERFACE CHANGES since Mail::SRS version 0.10.
+=item The cryptographic scheme is now HMAC with SHA1.
 
-This code has been written by Shevek <shevek@cpan.org> to emulate
-the functionality (but not the exact behaviour) of Mail::SRS-0.10
-from CPAN. The original documentation for Mail::SRS version 0.10
-is included for posterity in the files OLD-DOCS and README.pobox in
-this distribution.
+=item Only a prefix of the MAC is used.
 
-Implementors please note the warning about the adjustment of the HMAC
-hash below.
+This API is still a release candidate and should remain relatively
+stable.
 
 =head1 EXTENDING Mail::SRS
 
@@ -96,6 +94,24 @@ string. A string is promoted into an array of one element.
 The maximum number of days for which a timestamp is considered
 valid. After this time, the timestamp is invalid.
 
+=item HashLength
+
+The number of bytes of base64 encoded data to use for the cryptographic
+hash. More is better, but makes for longer addresses which might
+exceed the 64 character length suggested by RFC2821. This defaults to
+4, which gives 4 x 6 = 24 bits of cryptographic information, which
+means that a spammer will have to make 2^24 attempts to guarantee
+forging an SRS address.
+
+=item HashMin
+
+The shortest hash which we will allow to pass authentication. Since we
+allow any valid prefix of the full SHA1 HMAC to pass authentication,
+a spammer might just suggest a hash of length 0. We require at least
+HashMin characters, which must all be correct. Naturally, this must
+be no greater than HashLength and will default to HashLength unless
+otherwise specified.
+
 =back
 
 Some subclasses require other parameters. See their documentation for
@@ -115,8 +131,8 @@ sub new {
 	$self->{Secret} = [ $self->{Secret} ]
 					unless ref($self->{Secret}) eq 'ARRAY';
 	$self->{MaxAge} = 31 unless $self->{MaxAge};
-	warn "Overriding Separator is deprecated" if $self->{Separator};
-	$self->{Separator} = '+';	# This is fixed by the standard.
+	$self->{HashLength} = 4 unless $self->{HashLength};
+	$self->{HashMin} = $self->{HashLength} unless $self->{HashMin};
 	return bless $self, $class;
 }
 
@@ -182,8 +198,8 @@ sub reverse {
 This method, designed to be overridden by subclasses, takes as
 parameters the original host and user and must compile a new username
 for the SRS transformed address. It is expected that this new username
-will be joined on $self->separator, and will contain a hash generated
-from $self->hash_create(...), and possibly a timestamp generated by
+will be joined on $SRSSEP, and will contain a hash generated from
+$self->hash_create(...), and possibly a timestamp generated by
 $self->timestamp_create().
 
 =cut
@@ -193,37 +209,6 @@ sub compile {
 					"All subclasses override it";
 }
 
-sub old_compile {
-	my ($self, $sendhost, $senduser) = @_;
-
-	if ($senduser =~ m/^\Q$SRSTAG$self->{Separator}\E/io) {
-		# We just do the split because this was hashed with someone else's
-		# secret key and we can't check it.
-		# SRSTAG, hash, timestamp, host, user
-		my (undef, undef, undef, $oldhost, $olduser) =
-						split(qr/\Q$self->{Separator}\E/, $senduser, 5);
-		# We should do this sanity check. After all, it might NOT be
-		# an SRS address, unlikely though that is. We are in the presence
-		# of malicious agents. We can check more rigorously than this...
-		if (defined $oldhost and defined $olduser) {
-			$sendhost = $oldhost;
-			$senduser = $olduser;
-		}
-	}
-
-	my $timestamp = $self->timestamp_create();
-
-	# This has to be done in compile, because we might need access
-	# to it for storing in a database.
-	my $hash = $self->hash_create($timestamp, $sendhost, $senduser);
-
-	# Note that there are 5 fields here and that sendhost may
-	# not contain a + sign. Therefore, we do not need to escape
-	# + signs anywhere in order to reverse this transformation.
-	return join($self->separator,
-					$SRSTAG, $hash, $timestamp, $sendhost, $senduser);
-}
-
 =item $srs->parse($srsuser)
 
 This method, designed to be overridden by subclasses, takes an
@@ -239,29 +224,6 @@ sub parse {
 					"All subclasses override it";
 }
 
-sub old_parse {
-	my ($self, $user) = @_;
-
-	unless ($user =~ m/^\Q$SRSTAG$self->{Separator}\E/oi) {
-		die "Reverse address does not start with $SRSTAG.";
-	}
-
-	# The 5 here matches the number of fields we encoded above. If
-	# there are more + signs, then they belong in senduser anyway.
-	my (undef, $hash, $timestamp, $sendhost, $senduser) =
-					split(qr/\Q$self->{Separator}\E/, $user, 5);
-	# Again, this must match as above.
-	unless ($self->hash_verify($hash,$timestamp,$sendhost,$senduser)) {
-		die "Invalid hash";
-	}
-
-	unless ($self->timestamp_check($timestamp)) {
-		die "Invalid timestamp";
-	}
-
-	return ($sendhost, $senduser);
-}
-
 =item $srs->timestamp_create([$time])
 
 Return a two character timestamp representing 'today', or $time if
@@ -275,7 +237,7 @@ given. $time is a Unix timestamp (seconds since the aeon).
 # different expiry dates for different sources/targets, and we don't
 # have to store them.
 
-my @BASE64 = ('A'..'Z', 'a'..'z', '0'..'9', '=', '/');
+my @BASE64 = ('A'..'Z', 'a'..'z', '0'..'9', '+', '/');
 my %BASE64 = map { $BASE64[$_] => $_ } (0..63);
 my $TICKSLOTS = scalar(@BASE64) * scalar(@BASE64);	# Two chars
 my $PRECISION = 60 * 60 * 24;	# One day
@@ -341,12 +303,6 @@ not been tampered with. You must provide at least one piece of data
 to this method (otherwise this system is both cryptographically weak
 and there may be collision problems with sender addresses).
 
-WARNING: The digest is slightly modified: Digest::HMAC_MD5 uses +
-as one of the base64 characters. We cannot allow that since we use
-+ as our SRS separator. So we replace every + with a - and reverse
-this text transformation in hash_verify(). Our base64 character set
-is therefore 'A-Za-z0-9=/' instead of 'A-Za-z0-9+/'
-
 =cut
 
 sub hash_create {
@@ -355,13 +311,12 @@ sub hash_create {
 	my @secret = $self->get_secret;
 	croak "Cannot create a cryptographic MAC without a secret"
 					unless @secret;
-	my $hmac = new Digest::HMAC_MD5($secret[0]);
+	my $hmac = new Digest::HMAC_SHA1($secret[0]);
 	foreach (@args) {
 		$hmac->add($_);
 	}
 	my $hash = $hmac->b64digest;
-	$hash =~ s/\+/=/g;
-	return $hash;
+	return substr($hash, 0, $self->{HashLength});
 }
 
 =item $srs->hash_verify($hash, @data)
@@ -375,16 +330,16 @@ with an old secret.
 
 sub hash_verify {
 	my ($self, $hash, @args) = @_;
+	return undef unless length $hash >= $self->{HashMin};
 	my @secret = $self->get_secret;
 	croak "Cannot verify a cryptographic MAC without a secret"
 					unless @secret;
-	$hash =~ s/=/+/g;	# We have to put the +s back in the hash!
 	foreach my $secret (@secret) {
-		my $hmac = new Digest::HMAC_MD5($secret);
+		my $hmac = new Digest::HMAC_SHA1($secret);
 		foreach (@args) {
 			$hmac->add($_);
 		}
-		return 1 if $hmac->b64digest eq $hash;
+		return 1 if substr($hmac->b64digest, 0, length($hash)) eq $hash;
 	}
 	return undef;
 }
@@ -411,9 +366,46 @@ sub get_secret {
 	return @{$_[0]->{Secret}};
 }
 
-sub separator {
-	return $_[0]->{Separator};
-}
+=head1 EXPORTS
+
+Given :all, this module exports three variables.
+
+=item $SRSSEP
+
+The SRS separator. The choice of separator was fairly arbitrary. It
+cannot be any of the following:
+
+=over 4
+
+=item / +
+
+Used in Base64.
+
+=item -
+
+Used in domains.
+
+=item ! %
+
+Used in bang paths and source routing.
+
+=item :
+
+Cannot be used in a Windows NT or Apple filename.
+
+=item ; | *
+
+Shell or regular expression metacharacters are probably to be avoided.
+
+=back
+
+=item $SRSTAG
+
+The SRS0 tag.
+
+=item $SRSWRAP
+
+The SRS1 tag.
 
 =head1 BUGS
 
@@ -422,7 +414,7 @@ Email address parsing for quoted addresses is not yet done properly.
 =head1 SEE ALSO
 
 Mail::SRS::Guarded, Mail::SRS::DB, Mail::SRS::Reversable,
-"make teach", eg/*
+"make teach", eg/*, http://www.anarres.org/projects/srs/
 
 =head1 AUTHOR
 
